@@ -954,8 +954,7 @@ impl BitReader {
             shift += 7;
             debug_assert!(
                 shift <= MAX_VLQ_BYTE_LEN * 7,
-                "Num of bytes exceed MAX_VLQ_BYTE_LEN ({})",
-                MAX_VLQ_BYTE_LEN
+                "Num of bytes exceed MAX_VLQ_BYTE_LEN ({MAX_VLQ_BYTE_LEN})"
             );
             if likely(byte & 0x80 == 0) {
                 return Some(v);
@@ -1014,7 +1013,7 @@ mod tests {
     use crate::parquet::util::test_common::*;
 
     use rand::{
-        distributions::{Distribution, Standard},
+        distr::{Distribution, StandardUniform},
         Rng,
     };
     use std::fmt::Debug;
@@ -1326,8 +1325,7 @@ mod tests {
         (0..total).for_each(|i| {
             assert!(
                 writer.put_value(values[i], num_bits),
-                "[{}]: put_value() failed",
-                i
+                "[{i}]: put_value() failed"
             );
         });
 
@@ -1378,16 +1376,16 @@ mod tests {
         // test reading consecutively from a buffer
         let mut reader = BitReader::from(vec);
         let mut buffer = vec![0; NUM_BYTES];
-        let mut rng = rand::thread_rng();
+        let mut rng = rand::rng();
         let mut bits_read = 0;
 
         loop {
             if bits_read >= total_num_bits {
                 break;
             }
-            let n: usize = rng.gen();
+            let n: u64 = rng.random();
             let num_bits = n % 20;
-            bits_read += reader.get_bits(&mut buffer, bits_read, num_bits);
+            bits_read += reader.get_bits(&mut buffer, bits_read, num_bits as usize);
         }
 
         assert_eq!(total_num_bits, bits_read);
@@ -1424,14 +1422,14 @@ mod tests {
         }
 
         // test skipping consecutively
-        let mut rng = rand::thread_rng();
+        let mut rng = rand::rng();
         loop {
             if bits_read >= total_num_bits {
                 break;
             }
-            let n: usize = rng.gen();
+            let n: u64 = rng.random();
             let num_bits = n % 20;
-            bits_read += reader.skip_bits(num_bits);
+            bits_read += reader.skip_bits(num_bits as usize);
         }
 
         assert_eq!(total_num_bits, bits_read);
@@ -1479,8 +1477,7 @@ mod tests {
         for i in 0..batch.len() {
             assert_eq!(
                 batch[i], expected_values[i],
-                "num_bits = {}, index = {}",
-                num_bits, i
+                "num_bits = {num_bits}, index = {i}"
             );
         }
     }
@@ -1509,11 +1506,7 @@ mod tests {
                     reader.get_u32_batch(batch.as_mut_ptr(), *total, num_bits);
                 }
                 for i in 0..batch.len() {
-                    assert_eq!(
-                        batch[i], values[i],
-                        "num_bits = {}, index = {}",
-                        num_bits, i
-                    );
+                    assert_eq!(batch[i], values[i], "num_bits = {num_bits}, index = {i}");
                 }
             }
         }
@@ -1534,10 +1527,10 @@ mod tests {
     fn test_put_aligned_rand_numbers<T>(total: usize, num_bits: usize)
     where
         T: Copy + FromBytes + AsBytes + Debug + PartialEq,
-        Standard: Distribution<T>,
+        StandardUniform: Distribution<T>,
     {
         assert!(num_bits <= 32);
-        assert!(total % 2 == 0);
+        assert_eq!(total % 2, 0);
 
         let aligned_value_byte_width = std::mem::size_of::<T>();
         let value_byte_width = ceil(num_bits, 8);
@@ -1554,14 +1547,12 @@ mod tests {
             if i % 2 == 0 {
                 assert!(
                     writer.put_value(values[j] as u64, num_bits),
-                    "[{}]: put_value() failed",
-                    i
+                    "[{i}]: put_value() failed"
                 );
             } else {
                 assert!(
                     writer.put_aligned::<T>(aligned_values[j], aligned_value_byte_width),
-                    "[{}]: put_aligned() failed",
-                    i
+                    "[{i}]: put_aligned() failed"
                 );
             }
         }
@@ -1599,8 +1590,7 @@ mod tests {
         (0..total).for_each(|i| {
             assert!(
                 writer.put_vlq_int(values[i] as u64),
-                "[{}]; put_vlq_int() failed",
-                i
+                "[{i}]; put_vlq_int() failed"
             );
         });
 
@@ -1625,8 +1615,7 @@ mod tests {
         (0..total).for_each(|i| {
             assert!(
                 writer.put_zigzag_vlq_int(values[i] as i64),
-                "[{}]; put_zigzag_vlq_int() failed",
-                i
+                "[{i}]; put_zigzag_vlq_int() failed"
             );
         });
 
