@@ -40,12 +40,24 @@ impl From<prost::DecodeError> for ExpressionError {
     }
 }
 
+impl From<prost::UnknownEnumValue> for ExpressionError {
+    fn from(error: prost::UnknownEnumValue) -> ExpressionError {
+        ExpressionError::Deserialize(error.to_string())
+    }
+}
+
 impl From<prost::DecodeError> for ExecutionError {
     fn from(error: prost::DecodeError) -> ExecutionError {
         ExecutionError::DeserializeError(error.to_string())
     }
 }
 
+impl From<prost::UnknownEnumValue> for ExecutionError {
+    fn from(error: prost::UnknownEnumValue) -> ExecutionError {
+        ExecutionError::DeserializeError(error.to_string())
+    }
+}
+
 /// Deserialize bytes to protobuf type of expression
 pub fn deserialize_expr(buf: &[u8]) -> Result<spark_expression::Expr, ExpressionError> {
     match spark_expression::Expr::decode(&mut Cursor::new(buf)) {
