@@ -26,9 +26,9 @@ import org.apache.hadoop.fs.Path
 import org.apache.spark.sql.CometTestBase
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
 import org.apache.spark.sql.functions.{array, col, expr, lit, udf}
-import org.apache.spark.sql.types.StructType
 
-import org.apache.comet.CometSparkSessionExtensions.{isSpark34Plus, isSpark35Plus}
+import org.apache.comet.CometSparkSessionExtensions.{isSpark35Plus, isSpark40Plus}
+import org.apache.comet.serde.CometArrayExcept
 import org.apache.comet.testing.{DataGenOptions, ParquetGenerator}
 
 class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelper {
@@ -37,7 +37,7 @@ class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelp
     Seq(true, false).foreach { dictionaryEnabled =>
       withTempDir { dir =>
         val path = new Path(dir.toURI.toString, "test.parquet")
-        makeParquetFileAllTypes(path, dictionaryEnabled, 10000)
+        makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled, 10000)
         spark.read.parquet(path.toString).createOrReplaceTempView("t1")
         checkSparkAnswerAndOperator(
           sql("SELECT array_remove(array(_2, _3,_4), _2) from t1 where _2 is null"))
@@ -105,12 +105,7 @@ class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelp
           sql(s"SELECT array($fieldName, $fieldName) as a, $fieldName as b FROM t1")
             .createOrReplaceTempView("t2")
           val df = sql("SELECT array_remove(a, b) FROM t2")
-          field.dataType match {
-            case _: StructType =>
-            // skip due to https://github.com/apache/datafusion-comet/issues/1314
-            case _ =>
-              checkSparkAnswer(df)
-          }
+          checkSparkAnswer(df)
         }
       }
     }
@@ -119,7 +114,7 @@ class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelp
   test("array_remove - fallback for unsupported type struct") {
     withTempDir { dir =>
       val path = new Path(dir.toURI.toString, "test.parquet")
-      makeParquetFileAllTypes(path, dictionaryEnabled = true, 100)
+      makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled = true, 100)
       spark.read.parquet(path.toString).createOrReplaceTempView("t1")
       sql("SELECT array(struct(_1, _2)) as a, struct(_1, _2) as b FROM t1")
         .createOrReplaceTempView("t2")
@@ -135,12 +130,11 @@ class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelp
   }
 
   test("array_append") {
-    assume(isSpark34Plus)
     withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
       Seq(true, false).foreach { dictionaryEnabled =>
         withTempDir { dir =>
           val path = new Path(dir.toURI.toString, "test.parquet")
-          makeParquetFileAllTypes(path, dictionaryEnabled = dictionaryEnabled, 10000)
+          makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled = dictionaryEnabled, 10000)
           spark.read.parquet(path.toString).createOrReplaceTempView("t1");
           checkSparkAnswerAndOperator(spark.sql("Select array_append(array(_1),false) from t1"))
           checkSparkAnswerAndOperator(
@@ -161,61 +155,66 @@ class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelp
 
   test("array_prepend") {
     assume(isSpark35Plus) // in Spark 3.5 array_prepend is implemented via array_insert
-    Seq(true, false).foreach { dictionaryEnabled =>
-      withTempDir { dir =>
-        val path = new Path(dir.toURI.toString, "test.parquet")
-        makeParquetFileAllTypes(path, dictionaryEnabled = dictionaryEnabled, 10000)
-        spark.read.parquet(path.toString).createOrReplaceTempView("t1");
-        checkSparkAnswerAndOperator(spark.sql("Select array_prepend(array(_1),false) from t1"))
-        checkSparkAnswerAndOperator(
-          spark.sql("SELECT array_prepend(array(_2, _3, _4), 4) FROM t1"))
-        checkSparkAnswerAndOperator(
-          spark.sql("SELECT array_prepend(array(_2, _3, _4), null) FROM t1"));
-        checkSparkAnswerAndOperator(
-          spark.sql("SELECT array_prepend(array(_6, _7), CAST(6.5 AS DOUBLE)) FROM t1"));
-        checkSparkAnswerAndOperator(spark.sql("SELECT array_prepend(array(_8), 'test') FROM t1"));
-        checkSparkAnswerAndOperator(spark.sql("SELECT array_prepend(array(_19), _19) FROM t1"));
-        checkSparkAnswerAndOperator(
-          spark.sql("SELECT array_prepend((CASE WHEN _2 =_3 THEN array(_4) END), _4) FROM t1"));
+    withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
+      Seq(true, false).foreach { dictionaryEnabled =>
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "test.parquet")
+          makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled = dictionaryEnabled, 10000)
+          spark.read.parquet(path.toString).createOrReplaceTempView("t1");
+          checkSparkAnswerAndOperator(spark.sql("Select array_prepend(array(_1),false) from t1"))
+          checkSparkAnswerAndOperator(
+            spark.sql("SELECT array_prepend(array(_2, _3, _4), 4) FROM t1"))
+          checkSparkAnswerAndOperator(
+            spark.sql("SELECT array_prepend(array(_2, _3, _4), null) FROM t1"));
+          checkSparkAnswerAndOperator(
+            spark.sql("SELECT array_prepend(array(_6, _7), CAST(6.5 AS DOUBLE)) FROM t1"));
+          checkSparkAnswerAndOperator(
+            spark.sql("SELECT array_prepend(array(_8), 'test') FROM t1"));
+          checkSparkAnswerAndOperator(spark.sql("SELECT array_prepend(array(_19), _19) FROM t1"));
+          checkSparkAnswerAndOperator(
+            spark.sql("SELECT array_prepend((CASE WHEN _2 =_3 THEN array(_4) END), _4) FROM t1"));
+        }
       }
     }
   }
 
   test("ArrayInsert") {
-    assume(isSpark34Plus)
-    Seq(true, false).foreach(dictionaryEnabled =>
-      withTempDir { dir =>
-        val path = new Path(dir.toURI.toString, "test.parquet")
-        makeParquetFileAllTypes(path, dictionaryEnabled, 10000)
-        val df = spark.read
-          .parquet(path.toString)
-          .withColumn("arr", array(col("_4"), lit(null), col("_4")))
-          .withColumn("arrInsertResult", expr("array_insert(arr, 1, 1)"))
-          .withColumn("arrInsertNegativeIndexResult", expr("array_insert(arr, -1, 1)"))
-          .withColumn("arrPosGreaterThanSize", expr("array_insert(arr, 8, 1)"))
-          .withColumn("arrNegPosGreaterThanSize", expr("array_insert(arr, -8, 1)"))
-          .withColumn("arrInsertNone", expr("array_insert(arr, 1, null)"))
-        checkSparkAnswerAndOperator(df.select("arrInsertResult"))
-        checkSparkAnswerAndOperator(df.select("arrInsertNegativeIndexResult"))
-        checkSparkAnswerAndOperator(df.select("arrPosGreaterThanSize"))
-        checkSparkAnswerAndOperator(df.select("arrNegPosGreaterThanSize"))
-        checkSparkAnswerAndOperator(df.select("arrInsertNone"))
-      })
+    withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
+      Seq(true, false).foreach(dictionaryEnabled =>
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "test.parquet")
+          makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled, 10000)
+          val df = spark.read
+            .parquet(path.toString)
+            .withColumn("arr", array(col("_4"), lit(null), col("_4")))
+            .withColumn("arrInsertResult", expr("array_insert(arr, 1, 1)"))
+            .withColumn("arrInsertNegativeIndexResult", expr("array_insert(arr, -1, 1)"))
+            .withColumn("arrPosGreaterThanSize", expr("array_insert(arr, 8, 1)"))
+            .withColumn("arrNegPosGreaterThanSize", expr("array_insert(arr, -8, 1)"))
+            .withColumn("arrInsertNone", expr("array_insert(arr, 1, null)"))
+          checkSparkAnswerAndOperator(df.select("arrInsertResult"))
+          checkSparkAnswerAndOperator(df.select("arrInsertNegativeIndexResult"))
+          checkSparkAnswerAndOperator(df.select("arrPosGreaterThanSize"))
+          checkSparkAnswerAndOperator(df.select("arrNegPosGreaterThanSize"))
+          checkSparkAnswerAndOperator(df.select("arrInsertNone"))
+        })
+    }
   }
 
   test("ArrayInsertUnsupportedArgs") {
     // This test checks that the else branch in ArrayInsert
     // mapping to the comet is valid and fallback to spark is working fine.
-    assume(isSpark34Plus)
-    withTempDir { dir =>
-      val path = new Path(dir.toURI.toString, "test.parquet")
-      makeParquetFileAllTypes(path, dictionaryEnabled = false, 10000)
-      val df = spark.read
-        .parquet(path.toString)
-        .withColumn("arr", array(col("_4"), lit(null), col("_4")))
-        .withColumn("idx", udf((_: Int) => 1).apply(col("_4")))
-        .withColumn("arrUnsupportedArgs", expr("array_insert(arr, idx, 1)"))
-      checkSparkAnswer(df.select("arrUnsupportedArgs"))
+    withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
+      withTempDir { dir =>
+        val path = new Path(dir.toURI.toString, "test.parquet")
+        makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled = false, 10000)
+        val df = spark.read
+          .parquet(path.toString)
+          .withColumn("arr", array(col("_4"), lit(null), col("_4")))
+          .withColumn("idx", udf((_: Int) => 1).apply(col("_4")))
+          .withColumn("arrUnsupportedArgs", expr("array_insert(arr, idx, 1)"))
+        checkSparkAnswer(df.select("arrUnsupportedArgs"))
+      }
     }
   }
 
@@ -223,7 +222,7 @@ class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelp
     withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
       withTempDir { dir =>
         val path = new Path(dir.toURI.toString, "test.parquet")
-        makeParquetFileAllTypes(path, dictionaryEnabled = false, n = 10000)
+        makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled = false, n = 10000)
         spark.read.parquet(path.toString).createOrReplaceTempView("t1");
         checkSparkAnswerAndOperator(
           spark.sql("SELECT array_contains(array(_2, _3, _4), _2) FROM t1"))
@@ -233,15 +232,84 @@ class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelp
     }
   }
 
+  test("array_distinct") {
+    withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
+      Seq(true, false).foreach { dictionaryEnabled =>
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "test.parquet")
+          makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled, n = 10000)
+          spark.read.parquet(path.toString).createOrReplaceTempView("t1")
+          // The result needs to be in ascending order for checkSparkAnswerAndOperator to pass
+          // because datafusion array_distinct sorts the elements and then removes the duplicates
+          checkSparkAnswerAndOperator(
+            spark.sql("SELECT array_distinct(array(_2, _2, _3, _4, _4)) FROM t1"))
+          checkSparkAnswerAndOperator(
+            spark.sql("SELECT array_distinct((CASE WHEN _2 =_3 THEN array(_4) END)) FROM t1"))
+          checkSparkAnswerAndOperator(spark.sql(
+            "SELECT array_distinct((CASE WHEN _2 =_3 THEN array(_2, _2, _4, _4, _5) END)) FROM t1"))
+          // NULL needs to be the first element for checkSparkAnswerAndOperator to pass because
+          // datafusion array_distinct sorts the elements and then removes the duplicates
+          checkSparkAnswerAndOperator(
+            spark.sql(
+              "SELECT array_distinct(array(CAST(NULL AS INT), _2, _2, _3, _4, _4)) FROM t1"))
+          checkSparkAnswerAndOperator(spark.sql(
+            "SELECT array_distinct(array(CAST(NULL AS INT), CAST(NULL AS INT), _2, _2, _3, _4, _4)) FROM t1"))
+        }
+      }
+    }
+  }
+
+  test("array_union") {
+    withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
+      Seq(true, false).foreach { dictionaryEnabled =>
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "test.parquet")
+          makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled, n = 10000)
+          spark.read.parquet(path.toString).createOrReplaceTempView("t1")
+          checkSparkAnswerAndOperator(
+            spark.sql("SELECT array_union(array(_2, _3, _4), array(_3, _4)) FROM t1"))
+          checkSparkAnswerAndOperator(sql("SELECT array_union(array(_18), array(_19)) from t1"))
+          checkSparkAnswerAndOperator(spark.sql(
+            "SELECT array_union(array(CAST(NULL AS INT), _2, _3, _4), array(CAST(NULL AS INT), _2, _3)) FROM t1"))
+          checkSparkAnswerAndOperator(spark.sql(
+            "SELECT array_union(array(CAST(NULL AS INT), CAST(NULL AS INT), _2, _3, _4), array(CAST(NULL AS INT), CAST(NULL AS INT), _2, _3)) FROM t1"))
+        }
+      }
+    }
+  }
+
+  test("array_max") {
+    Seq(true, false).foreach { dictionaryEnabled =>
+      withTempDir { dir =>
+        val path = new Path(dir.toURI.toString, "test.parquet")
+        makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled, n = 10000)
+        spark.read.parquet(path.toString).createOrReplaceTempView("t1");
+        checkSparkAnswerAndOperator(spark.sql("SELECT array_max(array(_2, _3, _4)) FROM t1"))
+        checkSparkAnswerAndOperator(
+          spark.sql("SELECT array_max((CASE WHEN _2 =_3 THEN array(_4) END)) FROM t1"))
+        checkSparkAnswerAndOperator(
+          spark.sql("SELECT array_max((CASE WHEN _2 =_3 THEN array(_2, _4) END)) FROM t1"))
+        checkSparkAnswerAndOperator(
+          spark.sql("SELECT array_max(array(CAST(NULL AS INT), CAST(NULL AS INT))) FROM t1"))
+        checkSparkAnswerAndOperator(
+          spark.sql("SELECT array_max(array(_2, CAST(NULL AS INT))) FROM t1"))
+        checkSparkAnswerAndOperator(spark.sql("SELECT array_max(array()) FROM t1"))
+        checkSparkAnswerAndOperator(
+          spark.sql(
+            "SELECT array_max(array(double('-Infinity'), 0.0, double('Infinity'))) FROM t1"))
+      }
+    }
+  }
+
   test("array_intersect") {
     // https://github.com/apache/datafusion-comet/issues/1441
-    assume(!CometConf.isExperimentalNativeScan)
+    assume(!usingDataSourceExec)
     withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
 
       Seq(true, false).foreach { dictionaryEnabled =>
         withTempDir { dir =>
           val path = new Path(dir.toURI.toString, "test.parquet")
-          makeParquetFileAllTypes(path, dictionaryEnabled, 10000)
+          makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled, 10000)
           spark.read.parquet(path.toString).createOrReplaceTempView("t1")
           checkSparkAnswerAndOperator(
             sql("SELECT array_intersect(array(_2, _3, _4), array(_3, _4)) from t1"))
@@ -259,7 +327,7 @@ class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelp
       Seq(true, false).foreach { dictionaryEnabled =>
         withTempDir { dir =>
           val path = new Path(dir.toURI.toString, "test.parquet")
-          makeParquetFileAllTypes(path, dictionaryEnabled, 10000)
+          makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled, 10000)
           spark.read.parquet(path.toString).createOrReplaceTempView("t1")
           checkSparkAnswerAndOperator(sql(
             "SELECT array_join(array(cast(_1 as string), cast(_2 as string), cast(_6 as string)), ' @ ') from t1"))
@@ -280,7 +348,7 @@ class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelp
       Seq(true, false).foreach { dictionaryEnabled =>
         withTempDir { dir =>
           val path = new Path(dir.toURI.toString, "test.parquet")
-          makeParquetFileAllTypes(path, dictionaryEnabled, 10000)
+          makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled, 10000)
           spark.read.parquet(path.toString).createOrReplaceTempView("t1")
           checkSparkAnswerAndOperator(sql(
             "SELECT arrays_overlap(array(_2, _3, _4), array(_3, _4)) from t1 where _2 is not null"))
@@ -296,12 +364,13 @@ class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelp
   }
 
   test("array_compact") {
-    assume(isSpark34Plus)
+    // TODO fix for Spark 4.0.0
+    assume(!isSpark40Plus)
     withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
       Seq(true, false).foreach { dictionaryEnabled =>
         withTempDir { dir =>
           val path = new Path(dir.toURI.toString, "test.parquet")
-          makeParquetFileAllTypes(path, dictionaryEnabled = dictionaryEnabled, n = 10000)
+          makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled = dictionaryEnabled, n = 10000)
           spark.read.parquet(path.toString).createOrReplaceTempView("t1")
 
           checkSparkAnswerAndOperator(
@@ -315,4 +384,118 @@ class CometArrayExpressionSuite extends CometTestBase with AdaptiveSparkPlanHelp
     }
   }
 
+  test("array_except - basic test (only integer values)") {
+    withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
+      Seq(true, false).foreach { dictionaryEnabled =>
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "test.parquet")
+          makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled, 10000)
+          spark.read.parquet(path.toString).createOrReplaceTempView("t1")
+
+          checkSparkAnswerAndOperator(
+            sql("SELECT array_except(array(_2, _3, _4), array(_3, _4)) from t1"))
+          checkSparkAnswerAndOperator(sql("SELECT array_except(array(_18), array(_19)) from t1"))
+          checkSparkAnswerAndOperator(
+            spark.sql(
+              "SELECT array_except(array(_2, _2, _4), array(_4)) FROM t1 WHERE _2 IS NOT NULL"))
+        }
+      }
+    }
+  }
+
+  test("array_except - test all types (native Parquet reader)") {
+    withTempDir { dir =>
+      val path = new Path(dir.toURI.toString, "test.parquet")
+      val filename = path.toString
+      val random = new Random(42)
+      withSQLConf(CometConf.COMET_ENABLED.key -> "false") {
+        ParquetGenerator.makeParquetFile(
+          random,
+          spark,
+          filename,
+          100,
+          DataGenOptions(
+            allowNull = true,
+            generateNegativeZero = true,
+            generateArray = false,
+            generateStruct = false,
+            generateMap = false))
+      }
+      withSQLConf(CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
+        val table = spark.read.parquet(filename)
+        table.createOrReplaceTempView("t1")
+        // test with array of each column
+        val fields =
+          table.schema.fields.filter(field => CometArrayExcept.isTypeSupported(field.dataType))
+        for (field <- fields) {
+          val fieldName = field.name
+          val typeName = field.dataType.typeName
+          sql(
+            s"SELECT cast(array($fieldName, $fieldName) as array<$typeName>) as a, cast(array($fieldName) as array<$typeName>) as b FROM t1")
+            .createOrReplaceTempView("t2")
+          val df = sql("SELECT array_except(a, b) FROM t2")
+          checkSparkAnswerAndOperator(df)
+        }
+      }
+    }
+  }
+
+  test("array_except - test all types (convert from Parquet)") {
+    withTempDir { dir =>
+      val path = new Path(dir.toURI.toString, "test.parquet")
+      val filename = path.toString
+      val random = new Random(42)
+      withSQLConf(CometConf.COMET_ENABLED.key -> "false") {
+        val options = DataGenOptions(
+          allowNull = true,
+          generateNegativeZero = true,
+          generateArray = true,
+          generateStruct = true,
+          generateMap = false)
+        ParquetGenerator.makeParquetFile(random, spark, filename, 100, options)
+      }
+      withSQLConf(
+        CometConf.COMET_NATIVE_SCAN_ENABLED.key -> "false",
+        CometConf.COMET_SPARK_TO_ARROW_ENABLED.key -> "true",
+        CometConf.COMET_CONVERT_FROM_PARQUET_ENABLED.key -> "true",
+        CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true") {
+        val table = spark.read.parquet(filename)
+        table.createOrReplaceTempView("t1")
+        // test with array of each column
+        val fields =
+          table.schema.fields.filter(field => CometArrayExcept.isTypeSupported(field.dataType))
+        for (field <- fields) {
+          val fieldName = field.name
+          sql(s"SELECT array($fieldName, $fieldName) as a, array($fieldName) as b FROM t1")
+            .createOrReplaceTempView("t2")
+          val df = sql("SELECT array_except(a, b) FROM t2")
+          checkSparkAnswer(df)
+        }
+      }
+    }
+  }
+
+  test("array_repeat") {
+    withSQLConf(
+      CometConf.COMET_EXPR_ALLOW_INCOMPATIBLE.key -> "true",
+      CometConf.COMET_EXPLAIN_FALLBACK_ENABLED.key -> "true") {
+      Seq(true, false).foreach { dictionaryEnabled =>
+        withTempDir { dir =>
+          val path = new Path(dir.toURI.toString, "test.parquet")
+          makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled, 100)
+          spark.read.parquet(path.toString).createOrReplaceTempView("t1")
+
+          checkSparkAnswerAndOperator(sql("SELECT array_repeat(_4, null) from t1"))
+          checkSparkAnswerAndOperator(sql("SELECT array_repeat(_4, 0) from t1"))
+          checkSparkAnswerAndOperator(
+            sql("SELECT array_repeat(_2, 5) from t1 where _2 is not null"))
+          checkSparkAnswerAndOperator(sql("SELECT array_repeat(_2, 5) from t1 where _2 is null"))
+          checkSparkAnswerAndOperator(
+            sql("SELECT array_repeat(_3, _4) from t1 where _3 is not null"))
+          checkSparkAnswerAndOperator(sql("SELECT array_repeat(cast(_3 as string), 2) from t1"))
+          checkSparkAnswerAndOperator(sql("SELECT array_repeat(array(_2, _3, _4), 2) from t1"))
+        }
+      }
+    }
+  }
 }
