@@ -70,7 +70,7 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
           val data: Seq[(Float, Int)] = Seq((Float.PositiveInfinity, 1))
           withParquetTable(data, "tbl", false) {
             val df = sql("SELECT stddev_pop(_1), stddev_pop(_2) FROM tbl")
-            checkSparkAnswer(df)
+            checkSparkAnswerAndOperator(df)
           }
         }
       }
@@ -199,7 +199,7 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         makeParquetFile(path, 10000, 10, false)
         withParquetTable(path.toUri.toString, "tbl") {
           val df = sql("SELECT _g5 FROM tbl GROUP BY _g1, _g2, _g3, _g4, _g5")
-          checkSparkAnswer(df)
+          checkSparkAnswerAndOperator(df)
         }
       }
     }
@@ -216,7 +216,7 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
           makeParquetFile(path, 10000, 10, dictionaryEnabled)
           withParquetTable(path.toUri.toString, "tbl") {
             val df = sql("SELECT * FROM tbl").groupBy("_g1").agg(sum($"_3" + $"_g3"))
-            checkSparkAnswer(df)
+            checkSparkAnswerAndOperator(df)
           }
         }
       }
@@ -271,7 +271,8 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
           val path = new Path(dir.toURI.toString, "test")
           makeParquetFile(path, 10000, 10, dictionaryEnabled)
           withParquetTable(path.toUri.toString, "tbl") {
-            checkSparkAnswer(sql("SELECT * FROM tbl").sort("_g1").groupBy("_g1").agg(sum("_8")))
+            checkSparkAnswerAndOperator(
+              sql("SELECT * FROM tbl").sort("_g1").groupBy("_g1").agg(sum("_8")))
           }
         }
       }
@@ -323,7 +324,7 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         val df3 = sql("SELECT COUNT(_1), COUNT(_2) FROM tbl")
         checkAnswer(df3, Row(5, 5) :: Nil)
 
-        checkSparkAnswer("SELECT _2, MIN(_1), MAX(_1) FROM tbl GROUP BY _2")
+        checkSparkAnswerAndOperator("SELECT _2, MIN(_1), MAX(_1) FROM tbl GROUP BY _2")
       }
     }
   }
@@ -335,8 +336,8 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         "tbl",
         dictionaryEnabled) {
 
-        checkSparkAnswer("SELECT _2, AVG(_1) FROM tbl GROUP BY _2")
-        checkSparkAnswer("SELECT AVG(_2) FROM tbl")
+        checkSparkAnswerAndOperator("SELECT _2, AVG(_1) FROM tbl GROUP BY _2")
+        checkSparkAnswerAndOperator("SELECT AVG(_2) FROM tbl")
       }
     }
   }
@@ -348,10 +349,10 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         withTable(table) {
           sql(s"create table $table(col1 int, col2 int) using parquet")
           sql(s"insert into $table values(1, 1), (2, 1), (3, 2), (null, 2), (null, 1)")
-          checkSparkAnswer(s"SELECT COUNT(col1) FROM $table")
-          checkSparkAnswer(s"SELECT col2, COUNT(col1) FROM $table GROUP BY col2")
-          checkSparkAnswer(s"SELECT avg(col1) FROM $table")
-          checkSparkAnswer(s"SELECT col2, avg(col1) FROM $table GROUP BY col2")
+          checkSparkAnswerAndOperator(s"SELECT COUNT(col1) FROM $table")
+          checkSparkAnswerAndOperator(s"SELECT col2, COUNT(col1) FROM $table GROUP BY col2")
+          checkSparkAnswerAndOperator(s"SELECT avg(col1) FROM $table")
+          checkSparkAnswerAndOperator(s"SELECT col2, avg(col1) FROM $table GROUP BY col2")
         }
       }
     }
@@ -360,8 +361,8 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
   test("SUM/AVG non-decimal overflow") {
     Seq(true, false).foreach { dictionaryEnabled =>
       withParquetTable(Seq((0, 100.toLong), (0, Long.MaxValue)), "tbl", dictionaryEnabled) {
-        checkSparkAnswer("SELECT SUM(_2) FROM tbl GROUP BY _1")
-        checkSparkAnswer("SELECT AVG(_2) FROM tbl GROUP BY _1")
+        checkSparkAnswerAndOperator("SELECT SUM(_2) FROM tbl GROUP BY _1")
+        checkSparkAnswerAndOperator("SELECT AVG(_2) FROM tbl GROUP BY _1")
       }
     }
   }
@@ -373,8 +374,8 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         checkAnswer(df1, Row(0, 6) :: Row(1, 4) :: Nil)
         val df2 = sql("SELECT _2, COUNT(_1) FROM tbl GROUP BY _2")
         checkAnswer(df2, Row(0, 3) :: Row(1, 2) :: Nil)
-        checkSparkAnswer("SELECT _2, MIN(_1), MAX(_1) FROM tbl GROUP BY _2")
-        checkSparkAnswer("SELECT _2, AVG(_1) FROM tbl GROUP BY _2")
+        checkSparkAnswerAndOperator("SELECT _2, MIN(_1), MAX(_1) FROM tbl GROUP BY _2")
+        checkSparkAnswerAndOperator("SELECT _2, AVG(_1) FROM tbl GROUP BY _2")
       }
     }
   }
@@ -417,7 +418,7 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
 
         val df3 = sql("SELECT _2, MIN(_1), MAX(_1) FROM tbl GROUP BY _2")
         checkAnswer(df3, Row(null.asInstanceOf[Int], 0, 9) :: Row(1, 1, 7) :: Row(2, 2, 8) :: Nil)
-        checkSparkAnswer(sql("SELECT _2, AVG(_1) FROM tbl GROUP BY _2"))
+        checkSparkAnswerAndOperator(sql("SELECT _2, AVG(_1) FROM tbl GROUP BY _2"))
       }
     }
   }
@@ -441,7 +442,7 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         val df3 = sql("SELECT _2, MIN(_1), MAX(_1) FROM tbl GROUP BY _2")
         checkAnswer(df3, Row(null.asInstanceOf[Int], 0, 9) :: Row(1, 0, 7) :: Row(2, 0, 5) :: Nil)
 
-        checkSparkAnswer(sql("SELECT _2, AVG(_1) FROM tbl GROUP BY _2"))
+        checkSparkAnswerAndOperator(sql("SELECT _2, AVG(_1) FROM tbl GROUP BY _2"))
       }
     }
   }
@@ -469,7 +470,7 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
             Row(null.asInstanceOf[Int], null.asInstanceOf[Int], null.asInstanceOf[Int]),
             Row(1, null.asInstanceOf[Int], null.asInstanceOf[Int]),
             Row(2, null.asInstanceOf[Int], null.asInstanceOf[Int])))
-        checkSparkAnswer(sql("SELECT _2, SUM(_1) FROM tbl GROUP BY _2"))
+        checkSparkAnswerAndOperator(sql("SELECT _2, SUM(_1) FROM tbl GROUP BY _2"))
       }
     }
   }
@@ -480,9 +481,9 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         val path = new Path(dir.toURI.toString, "test")
         makeParquetFile(path, 1000, 10, dictionaryEnabled)
         withParquetTable(path.toUri.toString, "tbl") {
-          checkSparkAnswer(
+          checkSparkAnswerAndOperator(
             "SELECT _g5, SUM(_5), COUNT(_5), MIN(_5), MAX(_5), AVG(_5) FROM tbl GROUP BY _g5")
-          checkSparkAnswer(
+          checkSparkAnswerAndOperator(
             "SELECT _g6, SUM(_6), COUNT(_6), MIN(_6), MAX(_6), AVG(_6) FROM tbl GROUP BY _g6")
         }
       }
@@ -548,13 +549,13 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         makeParquetFile(path, 1000, 10, dictionaryEnabled)
         withParquetTable(path.toUri.toString, "tbl") {
           Seq("SUM", "AVG").foreach { FN =>
-            checkSparkAnswer(
+            checkSparkAnswerAndOperator(
               s"SELECT _g1, $FN(_8 + CAST(1 AS DECIMAL(20, 10))) FROM tbl GROUP BY _g1")
-            checkSparkAnswer(
+            checkSparkAnswerAndOperator(
               s"SELECT _g1, $FN(_8 - CAST(-1 AS DECIMAL(10, 3))) FROM tbl GROUP BY _g1")
-            checkSparkAnswer(
+            checkSparkAnswerAndOperator(
               s"SELECT _g1, $FN(_9 * CAST(3.14 AS DECIMAL(4, 3))) FROM tbl GROUP BY _g1")
-            checkSparkAnswer(
+            checkSparkAnswerAndOperator(
               s"SELECT _g1, $FN(_9 / CAST(1.2345 AS DECIMAL(35, 10))) FROM tbl GROUP BY _g1")
           }
         }
@@ -605,9 +606,12 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         val path = new Path(dir.toURI.toString, "test")
         makeParquetFile(path, 1000, 10, dictionaryEnabled)
         withParquetTable(path.toUri.toString, "tbl") {
-          checkSparkAnswer("SELECT _g1, COUNT(_10), MIN(_10), MAX(_10) FROM tbl GROUP BY _g1")
-          checkSparkAnswer("SELECT _g1, COUNT(_11), MIN(_11), MAX(_11) FROM tbl GROUP BY _g1")
-          checkSparkAnswer("SELECT _g1, COUNT(_12), MIN(_12), MAX(_12) FROM tbl GROUP BY _g1")
+          checkSparkAnswerAndOperator(
+            "SELECT _g1, COUNT(_10), MIN(_10), MAX(_10) FROM tbl GROUP BY _g1")
+          checkSparkAnswerAndOperator(
+            "SELECT _g1, COUNT(_11), MIN(_11), MAX(_11) FROM tbl GROUP BY _g1")
+          checkSparkAnswerAndOperator(
+            "SELECT _g1, COUNT(_12), MIN(_12), MAX(_12) FROM tbl GROUP BY _g1")
         }
       }
     }
@@ -629,7 +633,7 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
               dictionaryEnabled) {
               withView("v") {
                 sql("CREATE TEMP VIEW v AS SELECT _1, _2 FROM tbl ORDER BY _1")
-                checkSparkAnswer(
+                checkSparkAnswerAndOperator(
                   "SELECT _2, SUM(_1), SUM(DISTINCT _1), MIN(_1), MAX(_1), COUNT(_1)," +
                     " COUNT(DISTINCT _1), AVG(_1), FIRST(_1), LAST(_1) FROM v GROUP BY _2")
               }
@@ -656,13 +660,13 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
               withParquetTable(path.toUri.toString, "tbl") {
                 withView("v") {
                   sql("CREATE TEMP VIEW v AS SELECT _g1, _g2, _3 FROM tbl ORDER BY _3")
-                  checkSparkAnswer(
+                  checkSparkAnswerAndOperator(
                     "SELECT _g1, _g2, FIRST(_3) FROM v GROUP BY _g1, _g2 ORDER BY _g1, _g2")
-                  checkSparkAnswer(
+                  checkSparkAnswerAndOperator(
                     "SELECT _g1, _g2, LAST(_3) FROM v GROUP BY _g1, _g2 ORDER BY _g1, _g2")
-                  checkSparkAnswer(
+                  checkSparkAnswerAndOperator(
                     "SELECT _g1, _g2, FIRST(_3) IGNORE NULLS FROM v GROUP BY _g1, _g2 ORDER BY _g1, _g2")
-                  checkSparkAnswer(
+                  checkSparkAnswerAndOperator(
                     "SELECT _g1, _g2, LAST(_3) IGNORE NULLS FROM v GROUP BY _g1, _g2 ORDER BY _g1, _g2")
                 }
               }
@@ -732,9 +736,9 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
               withParquetTable(path.toUri.toString, "tbl") {
                 withView("v") {
                   sql("CREATE TEMP VIEW v AS SELECT _g3, _g4, _3, _4 FROM tbl ORDER BY _3, _4")
-                  checkSparkAnswer(
+                  checkSparkAnswerAndOperator(
                     "SELECT _g3, _g4, FIRST(_3), FIRST(_4) FROM v GROUP BY _g3, _g4 ORDER BY _g3, _g4")
-                  checkSparkAnswer(
+                  checkSparkAnswerAndOperator(
                     "SELECT _g3, _g4, LAST(_3), LAST(_4) FROM v GROUP BY _g3, _g4 ORDER BY _g3, _g4")
                 }
               }
@@ -799,8 +803,9 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
                   withView("v") {
                     sql(s"CREATE TEMP VIEW v AS SELECT _g$gCol, _1, _2, _3, _4 " +
                       "FROM tbl ORDER BY _1, _2, _3, _4")
-                    checkSparkAnswer(s"SELECT _g$gCol, FIRST(_1), FIRST(_2), FIRST(_3), " +
-                      s"FIRST(_4), LAST(_1), LAST(_2), LAST(_3), LAST(_4) FROM v GROUP BY _g$gCol ORDER BY _g$gCol")
+                    checkSparkAnswerAndOperator(
+                      s"SELECT _g$gCol, FIRST(_1), FIRST(_2), FIRST(_3), " +
+                        s"FIRST(_4), LAST(_1), LAST(_2), LAST(_3), LAST(_4) FROM v GROUP BY _g$gCol ORDER BY _g$gCol")
                   }
                 }
               }
@@ -811,6 +816,23 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     }
   }
 
+  test("first/last with ignore null") {
+    val data = Range(0, 8192).flatMap(n => Seq((n, 1), (n, 2))).toDF("a", "b")
+    withTempDir { dir =>
+      val filename = s"${dir.getAbsolutePath}/first_last_ignore_null.parquet"
+      data.write.parquet(filename)
+      withSQLConf(CometConf.COMET_BATCH_SIZE.key -> "100") {
+        spark.read.parquet(filename).createOrReplaceTempView("t1")
+        for (expr <- Seq("first", "last")) {
+          // deterministic query that should return one non-null value per group
+          val df = spark.sql(
+            s"SELECT a, $expr(IF(b==1,null,b)) IGNORE NULLS FROM t1 GROUP BY a ORDER BY a")
+          checkSparkAnswerAndOperator(df)
+        }
+      }
+    }
+  }
+
   test("all types, with nulls") {
     val numValues = 2048
 
@@ -936,6 +958,21 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     }
   }
 
+  test("sum overflow on decimal(38, _)") {
+    withSQLConf(CometConf.COMET_CAST_ALLOW_INCOMPATIBLE.key -> "true") {
+      val table = "overflow_decimal_38"
+      withTable(table) {
+        sql(s"create table $table(a decimal(38, 2), b INT) using parquet")
+        sql(s"insert into $table values(42.00, 1), (999999999999999999999999999999999999.99, 1)")
+        checkSparkAnswerAndNumOfAggregates(s"select sum(a) from $table", 2)
+        sql(s"insert into $table values(42.00, 2), (99999999999999999999999999999999.99, 2)")
+        sql(s"insert into $table values(999999999999999999999999999999999999.99, 3)")
+        sql(s"insert into $table values(99999999999999999999999999999999.99, 4)")
+        checkSparkAnswerAndNumOfAggregates(s"select sum(a) from $table group by b order by b", 2)
+      }
+    }
+  }
+
   test("test final avg") {
     withSQLConf(
       CometConf.COMET_EXEC_SHUFFLE_ENABLED.key -> "true",
@@ -1021,7 +1058,7 @@ class CometAggregateSuite extends CometTestBase with AdaptiveSparkPlanHelper {
         sql(s"insert into $table values(2, null)")
 
         val query = sql(s"select a, AVG(b) from $table GROUP BY a")
-        checkSparkAnswer(query)
+        checkSparkAnswerAndOperator(query)
       }
     }
   }
