@@ -32,9 +32,9 @@ import org.apache.commons.lang3.StringUtils
 import org.apache.hadoop.fs.Path
 import org.apache.parquet.column.ParquetProperties
 import org.apache.parquet.example.data.Group
-import org.apache.parquet.example.data.simple.SimpleGroup
+import org.apache.parquet.example.data.simple.{SimpleGroup, SimpleGroupFactory}
 import org.apache.parquet.hadoop.ParquetWriter
-import org.apache.parquet.hadoop.example.ExampleParquetWriter
+import org.apache.parquet.hadoop.example.{ExampleParquetWriter, GroupWriteSupport}
 import org.apache.parquet.schema.{MessageType, MessageTypeParser}
 import org.apache.spark._
 import org.apache.spark.internal.config.{MEMORY_OFFHEAP_ENABLED, MEMORY_OFFHEAP_SIZE, SHUFFLE_MANAGER}
@@ -44,10 +44,9 @@ import org.apache.spark.sql.execution._
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
 import org.apache.spark.sql.internal._
 import org.apache.spark.sql.test._
-import org.apache.spark.sql.types.{DecimalType, StructType}
+import org.apache.spark.sql.types.{ArrayType, DataType, DecimalType, MapType, StructType}
 
 import org.apache.comet._
-import org.apache.comet.CometSparkSessionExtensions.isSpark34Plus
 import org.apache.comet.shims.ShimCometSparkSessionExtensions
 
 /**
@@ -59,7 +58,8 @@ abstract class CometTestBase
     with SQLTestUtils
     with BeforeAndAfterEach
     with AdaptiveSparkPlanHelper
-    with ShimCometSparkSessionExtensions {
+    with ShimCometSparkSessionExtensions
+    with ShimCometTestBase {
   import testImplicits._
 
   protected val shuffleManager: String =
@@ -79,9 +79,13 @@ abstract class CometTestBase
     conf.set(CometConf.COMET_ENABLED.key, "true")
     conf.set(CometConf.COMET_EXEC_ENABLED.key, "true")
     conf.set(CometConf.COMET_EXEC_SHUFFLE_ENABLED.key, "true")
-    conf.set(CometConf.COMET_SHUFFLE_FALLBACK_TO_COLUMNAR.key, "true")
+    conf.set(CometConf.COMET_RESPECT_PARQUET_FILTER_PUSHDOWN.key, "true")
     conf.set(CometConf.COMET_SPARK_TO_ARROW_ENABLED.key, "true")
     conf.set(CometConf.COMET_NATIVE_SCAN_ENABLED.key, "true")
+    // set the scan impl to SCAN_NATIVE_COMET because many tests are implemented
+    // with the assumption that this is the default and would need updating if we
+    // change the default
+    conf.set(CometConf.COMET_NATIVE_SCAN_IMPL.key, CometConf.SCAN_NATIVE_COMET)
     conf.set(CometConf.COMET_SCAN_ALLOW_INCOMPATIBLE.key, "true")
     conf.set(CometConf.COMET_MEMORY_OVERHEAD.key, "2g")
     conf.set(CometConf.COMET_EXEC_SORT_MERGE_JOIN_WITH_JOIN_FILTER_ENABLED.key, "true")
@@ -120,7 +124,9 @@ abstract class CometTestBase
 
     actualAnswer.toSeq.zip(expectedAnswer.toSeq).foreach {
       case (actual: Double, expected: Double) =>
-        if (!actual.isNaN && !expected.isNaN) {
+        if (actual.isInfinity || expected.isInfinity) {
+          assert(actual.isInfinity == expected.isInfinity, s"actual answer $actual != $expected")
+        } else if (!actual.isNaN && !expected.isNaN) {
           assert(
             math.abs(actual - expected) < absTol,
             s"actual answer $actual not within $absTol of correct answer $expected")
@@ -145,11 +151,11 @@ abstract class CometTestBase
     var expected: Array[Row] = Array.empty
     var sparkPlan = null.asInstanceOf[SparkPlan]
     withSQLConf(CometConf.COMET_ENABLED.key -> "false") {
-      val dfSpark = Dataset.ofRows(spark, df.logicalPlan)
+      val dfSpark = datasetOfRows(spark, df.logicalPlan)
       expected = dfSpark.collect()
       sparkPlan = dfSpark.queryExecution.executedPlan
     }
-    val dfComet = Dataset.ofRows(spark, df.logicalPlan)
+    val dfComet = datasetOfRows(spark, df.logicalPlan)
     checkAnswer(dfComet, expected)
     (sparkPlan, dfComet.queryExecution.executedPlan)
   }
@@ -215,30 +221,31 @@ abstract class CometTestBase
   /**
    * Check the answer of a Comet SQL query with Spark result using absolute tolerance.
    */
-  protected def checkSparkAnswerWithTol(query: String, absTol: Double = 1e-6): Unit = {
+  protected def checkSparkAnswerWithTol(query: String, absTol: Double = 1e-6): DataFrame = {
     checkSparkAnswerWithTol(sql(query), absTol)
   }
 
   /**
    * Check the answer of a Comet DataFrame with Spark result using absolute tolerance.
    */
-  protected def checkSparkAnswerWithTol(df: => DataFrame, absTol: Double): Unit = {
+  protected def checkSparkAnswerWithTol(df: => DataFrame, absTol: Double): DataFrame = {
     var expected: Array[Row] = Array.empty
     withSQLConf(CometConf.COMET_ENABLED.key -> "false") {
-      val dfSpark = Dataset.ofRows(spark, df.logicalPlan)
+      val dfSpark = datasetOfRows(spark, df.logicalPlan)
       expected = dfSpark.collect()
     }
-    val dfComet = Dataset.ofRows(spark, df.logicalPlan)
+    val dfComet = datasetOfRows(spark, df.logicalPlan)
     checkAnswerWithTol(dfComet, expected, absTol: Double)
+    dfComet
   }
 
   protected def checkSparkMaybeThrows(
       df: => DataFrame): (Option[Throwable], Option[Throwable]) = {
     var expected: Option[Throwable] = None
     withSQLConf(CometConf.COMET_ENABLED.key -> "false") {
-      expected = Try(Dataset.ofRows(spark, df.logicalPlan).collect()).failed.toOption
+      expected = Try(datasetOfRows(spark, df.logicalPlan).collect()).failed.toOption
     }
-    val actual = Try(Dataset.ofRows(spark, df.logicalPlan).collect()).failed.toOption
+    val actual = Try(datasetOfRows(spark, df.logicalPlan).collect()).failed.toOption
     (expected, actual)
   }
 
@@ -249,10 +256,10 @@ abstract class CometTestBase
     var expected: Array[Row] = Array.empty
     var dfSpark: Dataset[Row] = null
     withSQLConf(CometConf.COMET_ENABLED.key -> "false", EXTENDED_EXPLAIN_PROVIDERS_KEY -> "") {
-      dfSpark = Dataset.ofRows(spark, df.logicalPlan)
+      dfSpark = datasetOfRows(spark, df.logicalPlan)
       expected = dfSpark.collect()
     }
-    val dfComet = Dataset.ofRows(spark, df.logicalPlan)
+    val dfComet = datasetOfRows(spark, df.logicalPlan)
     checkAnswer(dfComet, expected)
     if (checkExplainString) {
       val diff = StringUtils.difference(
@@ -274,8 +281,8 @@ abstract class CometTestBase
     }
   }
 
-  private var _spark: SparkSession = _
-  protected implicit def spark: SparkSession = _spark
+  private var _spark: SparkSessionType = _
+  override protected implicit def spark: SparkSessionType = _spark
   protected implicit def sqlContext: SQLContext = _spark.sqlContext
 
   override protected def sparkContext: SparkContext = {
@@ -294,8 +301,9 @@ abstract class CometTestBase
     SparkContext.getOrCreate(conf)
   }
 
-  protected def createSparkSession: SparkSession = {
-    SparkSession.cleanupAnyExistingSession()
+  protected def createSparkSession: SparkSessionType = {
+    SparkSession.clearActiveSession()
+    SparkSession.clearDefaultSession()
 
     SparkSession
       .builder()
@@ -428,134 +436,79 @@ abstract class CometTestBase
     value % div
   }
 
-  def makeParquetFileAllTypes(path: Path, dictionaryEnabled: Boolean, n: Int): Unit = {
-    makeParquetFileAllTypes(path, dictionaryEnabled, 0, n)
+  def makeParquetFileAllPrimitiveTypes(path: Path, dictionaryEnabled: Boolean, n: Int): Unit = {
+    makeParquetFileAllPrimitiveTypes(path, dictionaryEnabled, 0, n)
   }
 
   def getPrimitiveTypesParquetSchema: String = {
-    if (CometSparkSessionExtensions.usingDataFusionParquetExec(conf) &&
-      !CometConf.COMET_SCAN_ALLOW_INCOMPATIBLE.get(conf)) {
+    if (usingDataSourceExecWithIncompatTypes(conf)) {
       // Comet complex type reader has different behavior for uint_8, uint_16 types.
       // The issue stems from undefined behavior in the parquet spec and is tracked
       // here: https://github.com/apache/parquet-java/issues/3142
       // here: https://github.com/apache/arrow-rs/issues/7040
       // and here: https://github.com/apache/datafusion-comet/issues/1348
-      if (isSpark34Plus) {
-        """
-         |message root {
-         |  optional boolean                  _1;
-         |  optional int32                    _2(INT_8);
-         |  optional int32                    _3(INT_16);
-         |  optional int32                    _4;
-         |  optional int64                    _5;
-         |  optional float                    _6;
-         |  optional double                   _7;
-         |  optional binary                   _8(UTF8);
-         |  optional int32                    _9(UINT_32);
-         |  optional int32                    _10(UINT_32);
-         |  optional int32                    _11(UINT_32);
-         |  optional int64                    _12(UINT_64);
-         |  optional binary                   _13(ENUM);
-         |  optional FIXED_LEN_BYTE_ARRAY(3)  _14;
-         |  optional int32                    _15(DECIMAL(5, 2));
-         |  optional int64                    _16(DECIMAL(18, 10));
-         |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
-         |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
-         |  optional INT64                    _19(TIMESTAMP(MICROS,true));
-         |  optional INT32                    _20(DATE);
-         |  optional INT32                    _id;
-         |}
-        """.stripMargin
-      } else {
-        """
-         |message root {
-         |  optional boolean                  _1;
-         |  optional int32                    _2(INT_8);
-         |  optional int32                    _3(INT_16);
-         |  optional int32                    _4;
-         |  optional int64                    _5;
-         |  optional float                    _6;
-         |  optional double                   _7;
-         |  optional binary                   _8(UTF8);
-         |  optional int32                    _9(UINT_32);
-         |  optional int32                    _10(UINT_32);
-         |  optional int32                    _11(UINT_32);
-         |  optional int64                    _12(UINT_64);
-         |  optional binary                   _13(ENUM);
-         |  optional binary                   _14(UTF8);
-         |  optional int32                    _15(DECIMAL(5, 2));
-         |  optional int64                    _16(DECIMAL(18, 10));
-         |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
-         |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
-         |  optional INT64                    _19(TIMESTAMP(MICROS,true));
-         |  optional INT32                    _20(DATE);
-         |  optional INT32                    _id;
-         |}
-        """.stripMargin
-      }
+      """
+       |message root {
+       |  optional boolean                  _1;
+       |  optional int32                    _2(INT_8);
+       |  optional int32                    _3(INT_16);
+       |  optional int32                    _4;
+       |  optional int64                    _5;
+       |  optional float                    _6;
+       |  optional double                   _7;
+       |  optional binary                   _8(UTF8);
+       |  optional int32                    _9(UINT_32);
+       |  optional int32                    _10(UINT_32);
+       |  optional int32                    _11(UINT_32);
+       |  optional int64                    _12(UINT_64);
+       |  optional binary                   _13(ENUM);
+       |  optional FIXED_LEN_BYTE_ARRAY(3)  _14;
+       |  optional int32                    _15(DECIMAL(5, 2));
+       |  optional int64                    _16(DECIMAL(18, 10));
+       |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
+       |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
+       |  optional INT64                    _19(TIMESTAMP(MICROS,true));
+       |  optional INT32                    _20(DATE);
+       |  optional binary                   _21;
+       |  optional INT32                    _id;
+       |}
+      """.stripMargin
     } else {
-
-      if (isSpark34Plus) {
-        """
-         |message root {
-         |  optional boolean                  _1;
-         |  optional int32                    _2(INT_8);
-         |  optional int32                    _3(INT_16);
-         |  optional int32                    _4;
-         |  optional int64                    _5;
-         |  optional float                    _6;
-         |  optional double                   _7;
-         |  optional binary                   _8(UTF8);
-         |  optional int32                    _9(UINT_8);
-         |  optional int32                    _10(UINT_16);
-         |  optional int32                    _11(UINT_32);
-         |  optional int64                    _12(UINT_64);
-         |  optional binary                   _13(ENUM);
-         |  optional FIXED_LEN_BYTE_ARRAY(3)  _14;
-         |  optional int32                    _15(DECIMAL(5, 2));
-         |  optional int64                    _16(DECIMAL(18, 10));
-         |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
-         |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
-         |  optional INT64                    _19(TIMESTAMP(MICROS,true));
-         |  optional INT32                    _20(DATE);
-         |  optional INT32                    _id;
-         |}
-        """.stripMargin
-      } else {
-        """
-         |message root {
-         |  optional boolean                  _1;
-         |  optional int32                    _2(INT_8);
-         |  optional int32                    _3(INT_16);
-         |  optional int32                    _4;
-         |  optional int64                    _5;
-         |  optional float                    _6;
-         |  optional double                   _7;
-         |  optional binary                   _8(UTF8);
-         |  optional int32                    _9(UINT_8);
-         |  optional int32                    _10(UINT_16);
-         |  optional int32                    _11(UINT_32);
-         |  optional int64                    _12(UINT_64);
-         |  optional binary                   _13(ENUM);
-         |  optional binary                   _14(UTF8);
-         |  optional int32                    _15(DECIMAL(5, 2));
-         |  optional int64                    _16(DECIMAL(18, 10));
-         |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
-         |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
-         |  optional INT64                    _19(TIMESTAMP(MICROS,true));
-         |  optional INT32                    _20(DATE);
-         |  optional INT32                    _id;
-         |}
-        """.stripMargin
-      }
+      """
+       |message root {
+       |  optional boolean                  _1;
+       |  optional int32                    _2(INT_8);
+       |  optional int32                    _3(INT_16);
+       |  optional int32                    _4;
+       |  optional int64                    _5;
+       |  optional float                    _6;
+       |  optional double                   _7;
+       |  optional binary                   _8(UTF8);
+       |  optional int32                    _9(UINT_8);
+       |  optional int32                    _10(UINT_16);
+       |  optional int32                    _11(UINT_32);
+       |  optional int64                    _12(UINT_64);
+       |  optional binary                   _13(ENUM);
+       |  optional FIXED_LEN_BYTE_ARRAY(3)  _14;
+       |  optional int32                    _15(DECIMAL(5, 2));
+       |  optional int64                    _16(DECIMAL(18, 10));
+       |  optional FIXED_LEN_BYTE_ARRAY(16) _17(DECIMAL(38, 37));
+       |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
+       |  optional INT64                    _19(TIMESTAMP(MICROS,true));
+       |  optional INT32                    _20(DATE);
+       |  optional binary                   _21;
+       |  optional INT32                    _id;
+       |}
+      """.stripMargin
     }
   }
 
-  def makeParquetFileAllTypes(
+  def makeParquetFileAllPrimitiveTypes(
       path: Path,
       dictionaryEnabled: Boolean,
       begin: Int,
       end: Int,
+      nullEnabled: Boolean = true,
       pageSize: Int = 128,
       randomSize: Int = 0): Unit = {
     // alwaysIncludeUnsignedIntTypes means we include unsignedIntTypes in the test even if the
@@ -574,7 +527,7 @@ abstract class CometTestBase
 
     val rand = scala.util.Random
     val data = (begin until end).map { i =>
-      if (rand.nextBoolean()) {
+      if (nullEnabled && rand.nextBoolean()) {
         None
       } else {
         if (dictionaryEnabled) Some(i % 4) else Some(i)
@@ -604,7 +557,8 @@ abstract class CometTestBase
           record.add(17, i.toLong)
           record.add(18, i.toLong)
           record.add(19, i)
-          record.add(20, idGenerator.getAndIncrement())
+          record.add(20, i.toString)
+          record.add(21, idGenerator.getAndIncrement())
         case _ =>
       }
       writer.write(record)
@@ -632,7 +586,8 @@ abstract class CometTestBase
       record.add(17, i)
       record.add(18, i)
       record.add(19, i.toInt)
-      record.add(20, idGenerator.getAndIncrement())
+      record.add(20, i.toString)
+      record.add(21, idGenerator.getAndIncrement())
       writer.write(record)
     }
 
@@ -746,6 +701,131 @@ abstract class CometTestBase
     expected
   }
 
+  // Generate a file based on a complex schema. Schema derived from https://arrow.apache.org/blog/2022/10/17/arrow-parquet-encoding-part-3/
+  def makeParquetFileComplexTypes(
+      path: Path,
+      dictionaryEnabled: Boolean,
+      numRows: Integer = 10000): Unit = {
+    val schemaString =
+      """
+      message ComplexDataSchema {
+        optional group optional_array (LIST) {
+          repeated group list {
+            optional int32 element;
+          }
+        }
+        required group array_of_struct (LIST) {
+          repeated group list {
+            optional group struct_element {
+              required int32 field1;
+              optional group optional_nested_array (LIST) {
+                repeated group list {
+                  required int32 element;
+                }
+              }
+            }
+          }
+        }
+        optional group optional_map (MAP) {
+          repeated group key_value {
+            required int32 key;
+            optional int32 value;
+          }
+        }
+        required group complex_map (MAP) {
+          repeated group key_value {
+            required group map_key {
+              required int32 key_field1;
+              optional int32 key_field2;
+            }
+            required group map_value {
+              required int32 value_field1;
+              repeated int32 value_field2;
+            }
+          }
+        }
+      }
+    """
+
+    val schema: MessageType = MessageTypeParser.parseMessageType(schemaString)
+    GroupWriteSupport.setSchema(schema, spark.sparkContext.hadoopConfiguration)
+
+    val writer = createParquetWriter(schema, path, dictionaryEnabled)
+
+    val groupFactory = new SimpleGroupFactory(schema)
+
+    for (i <- 0 until numRows) {
+      val record = groupFactory.newGroup()
+
+      // Optional array of optional integers
+      if (i % 2 == 0) { // optional_array for every other row
+        val optionalArray = record.addGroup("optional_array")
+        for (j <- 0 until (i % 5)) {
+          val elementGroup = optionalArray.addGroup("list")
+          if (j % 2 == 0) { // some elements are optional
+            elementGroup.append("element", j)
+          }
+        }
+      }
+
+      // Required array of structs
+      val arrayOfStruct = record.addGroup("array_of_struct")
+      for (j <- 0 until (i % 3) + 1) { // Add one to three elements
+        val structElementGroup = arrayOfStruct.addGroup("list").addGroup("struct_element")
+        structElementGroup.append("field1", i * 10 + j)
+
+        // Optional nested array
+        if (j % 2 != 0) { // optional nested array for every other struct
+          val nestedArray = structElementGroup.addGroup("optional_nested_array")
+          for (k <- 0 until (i % 4)) { // Add zero to three elements.
+            val nestedElementGroup = nestedArray.addGroup("list")
+            nestedElementGroup.append("element", i + j + k)
+          }
+        }
+      }
+
+      // Optional map
+      if (i % 3 == 0) { // optional_map every third row
+        val optionalMap = record.addGroup("optional_map")
+        optionalMap
+          .addGroup("key_value")
+          .append("key", i)
+          .append("value", i)
+        if (i % 5 == 0) { // another optional entry
+          optionalMap
+            .addGroup("key_value")
+            .append("key", i)
+          // Value is optional
+          if (i % 10 == 0) {
+            optionalMap
+              .addGroup("key_value")
+              .append("key", i)
+              .append("value", i)
+          }
+        }
+      }
+
+      // Required map with complex key and value types
+      val complexMap = record.addGroup("complex_map")
+      val complexMapKeyVal = complexMap.addGroup("key_value")
+
+      complexMapKeyVal
+        .addGroup("map_key")
+        .append("key_field1", i)
+
+      complexMapKeyVal
+        .addGroup("map_value")
+        .append("value_field1", i)
+        .append("value_field2", i * 100)
+        .append("value_field2", i * 101)
+        .append("value_field2", i * 102)
+
+      writer.write(record)
+    }
+
+    writer.close()
+  }
+
   protected def makeDateTimeWithFormatTable(
       path: Path,
       dictionaryEnabled: Boolean,
@@ -865,13 +945,58 @@ abstract class CometTestBase
     }
   }
 
-  // tests one liner query without necessity to create external table
+  /**
+   * The test encapsulates integration Comet test and does following:
+   *   - prepares data using SELECT query and saves it to the Parquet file in temp folder
+   *   - creates a temporary table with name `tableName` on top of temporary parquet file
+   *   - runs the query `testQuery` reading data from `tableName`
+   *
+   * Asserts the `testQuery` data with Comet is the same is with Apache Spark and also asserts
+   * only Comet operator are in the physical plan
+   *
+   * Example:
+   *
+   * {{{
+   *  test("native reader - read simple ARRAY fields with SHORT field") {
+   *     testSingleLineQuery(
+   *       """
+   *         |select array(cast(1 as short)) arr
+   *         |""".stripMargin,
+   *       "select arr from tbl",
+   *       sqlConf = Seq(
+   *         CometConf.COMET_SCAN_ALLOW_INCOMPATIBLE.key -> "false",
+   *         "spark.comet.explainFallback.enabled" -> "false"
+   *       ),
+   *       debugCometDF = df => {
+   *         df.printSchema()
+   *         df.explain("extended")
+   *         df.show()
+   *       })
+   *   }
+   * }}}
+   *
+   * @param prepareQuery
+   *   prepare sample data with Comet disabled
+   * @param testQuery
+   *   the query to test. Typically with Comet enabled + other SQL config applied
+   * @param testName
+   *   test name
+   * @param tableName
+   *   table name where sample data stored
+   * @param sqlConf
+   *   additional spark sql configuration
+   * @param debugCometDF
+   *   optional debug access to DataFrame for `testQuery`
+   */
   def testSingleLineQuery(
       prepareQuery: String,
       testQuery: String,
       testName: String = "test",
       tableName: String = "tbl",
-      sqlConf: Seq[(String, String)] = Seq.empty): Unit = {
+      sqlConf: Seq[(String, String)] = Seq.empty,
+      readSchema: Option[StructType] = None,
+      debugCometDF: DataFrame => Unit = _ => (),
+      checkCometOperator: Boolean = true): Unit = {
 
     withTempDir { dir =>
       val path = new Path(dir.toURI.toString, testName).toUri.toString
@@ -885,10 +1010,15 @@ abstract class CometTestBase
       }
 
       spark.createDataFrame(data, schema).repartition(1).write.parquet(path)
-      readParquetFile(path, Some(schema)) { df => df.createOrReplaceTempView(tableName) }
+      readParquetFile(path, readSchema.orElse(Some(schema))) { df =>
+        df.createOrReplaceTempView(tableName)
+      }
 
       withSQLConf(sqlConf: _*) {
-        checkSparkAnswerAndOperator(sql(testQuery))
+        val cometDF = sql(testQuery)
+        debugCometDF(cometDF)
+        if (checkCometOperator) checkSparkAnswerAndOperator(cometDF)
+        else checkSparkAnswer(cometDF)
       }
     }
   }
@@ -987,4 +1117,20 @@ abstract class CometTestBase
 
     writer.close()
   }
+
+  def usingDataSourceExec: Boolean = usingDataSourceExec(SQLConf.get)
+
+  def usingDataSourceExec(conf: SQLConf): Boolean =
+    Seq(CometConf.SCAN_NATIVE_ICEBERG_COMPAT, CometConf.SCAN_NATIVE_DATAFUSION).contains(
+      CometConf.COMET_NATIVE_SCAN_IMPL.get(conf))
+
+  def usingDataSourceExecWithIncompatTypes(conf: SQLConf): Boolean = {
+    usingDataSourceExec(conf) &&
+    !CometConf.COMET_SCAN_ALLOW_INCOMPATIBLE.get(conf)
+  }
+
+  def isComplexType(dt: DataType): Boolean = dt match {
+    case _: StructType | _: ArrayType | _: MapType => true
+    case _ => false
+  }
 }
